'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.mapAsyncIterable = void 0;
/**
 * Given an AsyncIterable and a callback function, return an AsyncIterator
 * which produces values mapped via calling the callback function.
 */
function mapAsyncIterable(iterable, valueCallback, finallyCallback) {
  const iterator = iterable[Symbol.asyncIterator]();
  async function mapResult(result) {
    if (result.done) {
      finallyCallback();
      return result;
    }
    try {
      return { value: await valueCallback(result.value), done: false };
    } catch (error) {
      /* c8 ignore start */
      // FIXME: add test case
      if (typeof iterator.return === 'function') {
        try {
          await iterator.return();
        } catch (_e) {
          /* ignore error */
        }
      }
      throw error;
      /* c8 ignore stop */
    }
  }
  return {
    async next() {
      return mapResult(await iterator.next());
    },
    async return() {
      // If iterator.return() does not exist, then type R must be undefined.
      const result =
        typeof iterator.return === 'function'
          ? await iterator.return()
          : { value: undefined, done: true };
      return mapResult(result);
    },
    async throw(error) {
      if (typeof iterator.throw === 'function') {
        return mapResult(await iterator.throw(error));
      }
      finallyCallback();
      throw error;
    },
    [Symbol.asyncIterator]() {
      return this;
    },
  };
}
exports.mapAsyncIterable = mapAsyncIterable;
